from transformers import PretrainedConfig
from transformers import PreTrainedModel
from transformers import AutoModel
from transformers import AutoTokenizer
from transformers import AutoConfig
from transformers import AutoModelForSequenceClassification
import torch
from torch.nn import functional as F
import transformers
from .configuration_ann import AnnConfig

class AnnForTabularRegression(PreTrainedModel):
    config_class = AnnConfig

    def __init__(self, config):
        super().__init__(config)
        self.layer = torch.nn.Sequential(
            torch.nn.Linear(in_features=config.in_features, out_features=32),
            #torch.nn.BatchNorm1d(num_features=32),
            torch.nn.ReLU(),
            torch.nn.Linear(in_features=32, out_features=1)   
        )

    def forward(self, x, labels=None):
        #print(x.shape) #
        logits = self.layer(x)
        #print(logits.shape) #
        if labels == None:
            return transformers.file_utils.ModelOutput({'logits': logits})
        else:
            #print(labels.shape) #torch.Size([8, 1]) #
            #loss = torch.nn.MSELoss()(logits, labels) 
            loss = F.mse_loss(logits, labels) 
            return transformers.file_utils.ModelOutput({'loss': loss, 'logits': logits})

'''
#오토 모델에 등록
AutoModelForTabularRegression.register(AnnConfig, AnnForTabularRegression)
'''

####################

from transformers import PretrainedConfig
from transformers import PreTrainedModel
from transformers import AutoModel
from transformers import AutoTokenizer
from transformers import AutoConfig
from transformers import AutoModelForSequenceClassification
import transformers
import torch
from torch.nn import functional as F
import transformers
from .configuration_ann import AnnConfig

class AnnForTabularBinaryClassification(PreTrainedModel):
    config_class = AnnConfig

    def __init__(self, config):
        super().__init__(config)
        self.layer = torch.nn.Sequential(
            torch.nn.Linear(in_features=config.in_features, out_features=32),
            #torch.nn.BatchNorm1d(num_features=32),
            torch.nn.ReLU(),
            torch.nn.Linear(in_features=32, out_features=1)
        )

    def forward(self, x, labels=None):
        #print(x.shape) #torch.Size([2, 4])
        logits = self.layer(x)
        #print(logits.shape) #torch.Size([16, 3])
        if labels == None:
            return transformers.file_utils.ModelOutput({'logits': logits})
        else:
            #print(labels.shape) #torch.Size([2, 3])
            #loss = F.binary_cross_entropy(F.sigmoid(logits), labels)
            loss = F.binary_cross_entropy_with_logits(logits, labels)
            return transformers.file_utils.ModelOutput({'loss': loss, 'logits': logits})

'''
#오토 모델에 등록
AutoModelForTabularClassification.register(AnnConfig, AnnForTabularBinaryClassification)
'''

####################

from transformers import PretrainedConfig
from transformers import PreTrainedModel
from transformers import AutoModel
from transformers import AutoTokenizer
from transformers import AutoConfig
from transformers import AutoModelForSequenceClassification
import torch
from torch.nn import functional as F
import transformers
from .configuration_ann import AnnConfig

class AnnForTabularClassification(PreTrainedModel):
    config_class = AnnConfig

    def __init__(self, config):
        super().__init__(config)
        self.layer = torch.nn.Sequential(
            torch.nn.Linear(in_features=config.in_features, out_features=32),
            #torch.nn.BatchNorm1d(num_features=32),
            torch.nn.ReLU(),
            torch.nn.Linear(in_features=32, out_features=config.num_labels)
        )

    def forward(self, x, labels=None):
        #print(x.shape) #torch.Size([2, 4])
        logits = self.layer(x)
        #print(logits.shape) #torch.Size([16, 3])
        if labels == None:
            return transformers.file_utils.ModelOutput({'logits': logits})
        else:
            #print(labels.shape) #torch.Size([2, 3])
            loss = F.nll_loss(F.log_softmax(logits), labels) #원핫 벡터를 넣을 필요없이 바로 실제값을 인자로 사용 #nll은 Negative Log Likelihood의 약자
            return transformers.file_utils.ModelOutput({'loss': loss, 'logits': logits})

'''
#오토 모델에 등록
AutoModelForTabularClassification.register(AnnConfig, AnnForTabularClassification)
'''

####################



from transformers import PretrainedConfig
from transformers import PreTrainedModel
from transformers import AutoModel
from transformers import AutoTokenizer
from transformers import AutoConfig
from transformers import AutoModelForSequenceClassification
import torch
from torch.nn import functional as F
import pytorch_supporter
import transformers

class EmbeddedRnnConfig(PretrainedConfig):
    model_type = "embedded-rnn"

    def __init__(self, **kwargs):
        #https://github.com/huggingface/transformers/blob/98d88b23f54e5a23e741833f1e973fdf600cc2c5/src/transformers/configuration_utils.py#L323
        #Keys are always strings in JSON so convert ids to int here.       
        super().__init__(**kwargs)

#오토 설정에 등록
model_path = 'embedded-rnn'
AutoConfig.register(model_path, EmbeddedRnnConfig)

class PretrainedEmbeddedRnnConfig(PretrainedConfig):
    model_type = "pretrained-embedded-rnn"

    def __init__(self, **kwargs):
        #https://github.com/huggingface/transformers/blob/98d88b23f54e5a23e741833f1e973fdf600cc2c5/src/transformers/configuration_utils.py#L323
        #Keys are always strings in JSON so convert ids to int here.       
        super().__init__(**kwargs)

#오토 설정에 등록
model_path = 'pretrained-embedded-rnn'
AutoConfig.register(model_path, PretrainedEmbeddedRnnConfig)



from transformers import PretrainedConfig
from transformers import PreTrainedModel
from transformers import AutoModel
from transformers import AutoConfig
from transformers import AutoModelForObjectDetection
import torch
import transformers
import torchvision
from torchvision.models.detection.faster_rcnn import FastRCNNPredictor
from torchvision.models.detection.faster_rcnn import FasterRCNN_ResNet50_FPN_Weights
from torchvision.models.detection import FasterRCNN
from torchvision.models.detection.rpn import AnchorGenerator
from torchvision.models.detection._utils import Matcher
from torchvision.ops.boxes import box_iou
from .configuration_faster_rcnn import FasterRcnnConfig

class FasterRcnnForObjectDetection(PreTrainedModel):
    config_class = FasterRcnnConfig

    def __init__(self, config):
        super().__init__(config)
        # load a model; pre-trained on COCO
        weights = FasterRCNN_ResNet50_FPN_Weights.COCO_V1 
        self.model = torchvision.models.detection.fasterrcnn_resnet50_fpn(weights=weights)
        # get number of input features for the classifier
        in_features = self.model.roi_heads.box_predictor.cls_score.in_features
        #num_classes = 2  # 1 class (early_printed_illustration) + background
        # replace the pre-trained head with a new one
        self.model.roi_heads.box_predictor = FastRCNNPredictor(in_features, config.num_labels)

    def forward(self, pixel_values, labels=None):
        if labels == None:
            outputs = self.model(pixel_values)
            boxes = []
            labels = []
            scores = []
            for output in outputs:
                boxes.append(output['boxes'])
                labels.append(output['labels'])
                scores.append(output['scores'])
            return transformers.file_utils.ModelOutput({'boxes': boxes, 'labels': labels, 'scores': scores})
        else:
            if self.training: #train
                loss_dict = self.model(pixel_values, labels)
                #print(loss_dict) #[{'boxes': tensor([], device='cuda:0', size=(0, 4)), 'labels': tensor([], device='cuda:0', dtype=torch.int64), 'scores': tensor([], device='cuda:0')}, {'boxes': tensor([], device='cuda:0', size=(0, 4)), 'labels': tensor([], device='cuda:0', dtype=torch.int64), 'scores': tensor([], device='cuda:0')}]
                #print(loss_dict.keys()) #dict_keys(['loss_classifier', 'loss_box_reg', 'loss_objectness', 'loss_rpn_box_reg'])
                #print(loss_dict.values()) #dict_values([tensor(0.7016, grad_fn=<NllLossBackward0>), tensor(0.6681, grad_fn=<DivBackward0>), tensor(4.9426, grad_fn=<BinaryCrossEntropyWithLogitsBackward0>), tensor(0.1818, dtype=torch.float64, grad_fn=<DivBackward0>)])
                loss = sum([loss for loss in loss_dict.values()])
                self.model.eval()
                outputs = self.model(pixel_values)
                boxes = []
                labels = []
                scores = []
                for output in outputs:
                    boxes.append(output['boxes'])
                    labels.append(output['labels'])
                    scores.append(output['scores'])
                self.model.train()
                return transformers.file_utils.ModelOutput({'loss': loss, 'boxes': boxes, 'labels': labels, 'scores': scores})
            else: #validation
                self.model.train()
                loss_dict = self.model(pixel_values, labels)
                loss = sum([loss for loss in loss_dict.values()])
                self.model.eval()
                outputs = self.model(pixel_values)
                boxes = []
                labels = []
                scores = []
                for output in outputs:
                    boxes.append(output['boxes'])
                    labels.append(output['labels'])
                    scores.append(output['scores'])
                return transformers.file_utils.ModelOutput({'loss': loss, 'boxes': boxes, 'labels': labels, 'scores': scores})

#오토 모델에 등록
AutoModelForObjectDetection.register(FasterRcnnConfig, FasterRcnnForObjectDetection)




from transformers import Pipeline
from transformers.pipelines import PIPELINE_REGISTRY
from torch.nn import functional as F

class FixedLengthTranslationPipeline(Pipeline):
    def _sanitize_parameters(self, **kwargs):
        preprocess_kwargs = {}
        if 'vocab_path' in kwargs:
            preprocess_kwargs['vocab_path'] = kwargs['vocab_path']
        postprocess_kwargs = {}
        return preprocess_kwargs, {}, postprocess_kwargs

    def preprocess(self, inputs, vocab_path=None):
        return self.tokenizer(inputs, vocab_path=vocab_path, return_tensors=self.framework)        

    def _forward(self, model_inputs):
        return self.model(**model_inputs)

    def postprocess(self, model_outputs):
        logits = model_outputs['logits']
        #print(logits.shape) #torch.Size([1, 3, 9])

        probabilities = F.softmax(logits, dim=-1)
        probabilities = probabilities.argmax(axis=-1)
        results = []
        for probability in probabilities:
            #print(probability) #tensor([2, 4, 6])
            tokens = self.tokenizer.convert_ids_to_tokens(probability)
            translation_text = ' '.join(tokens)
            #print(translation_text) #i raise dog
            results.append({'translation_text': translation_text})         
        return results

PIPELINE_REGISTRY.register_pipeline('fixed-length-translation', 
                                    #pt_model=AutoModelForFixedLengthTranslation,
                                    pipeline_class=FixedLengthTranslationPipeline)



from transformers import PretrainedConfig
from transformers import PreTrainedModel
from transformers import AutoModel
from transformers import AutoTokenizer
from transformers import AutoConfig
from transformers import AutoModelForAudioClassification
import pytorch_supporter
import torch
from torch.nn import functional as F
import transformers
from .configuration_rnn import RnnConfig

class RnnForAudioClassification(PreTrainedModel):
    config_class = RnnConfig

    def __init__(self, config):
        super().__init__(config)        
        self.layer = torch.nn.Sequential(
            pytorch_supporter.layers.HiddenStateResetLSTM(input_size=1, hidden_size=32, batch_first=True),
            pytorch_supporter.layers.LSTMLastHiddenState(),
            torch.nn.Linear(in_features=32, out_features=config.num_labels)
        )

    def forward(self, input_values, labels=None):
        if len(input_values.shape) == 2:
            input_values = input_values.unsqueeze(-1)
        
        logits = self.layer(input_values)
        #print(logits.shape) #torch.Size([16, 3])
        if labels == None:
            return transformers.file_utils.ModelOutput({'logits': logits})
        else:
            loss = F.nll_loss(F.log_softmax(logits), labels) #원핫 벡터를 넣을 필요없이 바로 실제값을 인자로 사용 #nll은 Negative Log Likelihood의 약자
            return transformers.file_utils.ModelOutput({'loss': loss, 'logits': logits})

#모델을 오토 모델에 등록
model_path = 'rnn-config'
AutoConfig.register(model_path, RnnConfig)
AutoModelForAudioClassification.register(RnnConfig, RnnForAudioClassification)

####################

from transformers import PretrainedConfig
from transformers import PreTrainedModel
from transformers import AutoModel
from transformers import AutoTokenizer
from transformers import AutoConfig
from transformers import AutoModelForAudioClassification
import pytorch_supporter
import torch
from torch.nn import functional as F
import transformers
from .configuration_rnn import RnnConfig

class RnnForTimeSeriesRegression(PreTrainedModel):
    config_class = RnnConfig

    def __init__(self, config):
        super().__init__(config)        
        self.layer = torch.nn.Sequential(
            pytorch_supporter.layers.HiddenStateResetLSTM(input_size=1, hidden_size=32, batch_first=True),
            pytorch_supporter.layers.LSTMLastHiddenState(),
            torch.nn.Linear(in_features=32, out_features=32),
            #torch.nn.BatchNorm1d(num_features=32),
            torch.nn.ReLU(),
            torch.nn.Linear(in_features=32, out_features=1)   
        )

    def forward(self, input_values, labels=None):
        if len(input_values.shape) == 2:
            input_values = input_values.unsqueeze(-1)
        
        logits = self.layer(input_values)
        #print(logits.shape) #torch.Size([16, 3])
        if labels == None:
            return transformers.file_utils.ModelOutput({'logits': logits})
        else:
            #print(labels.shape) #torch.Size([8, 1]) #
            #loss = torch.nn.MSELoss()(logits, labels) 
            loss = F.mse_loss(logits, labels) 
            return transformers.file_utils.ModelOutput({'loss': loss, 'logits': logits})
        
'''
#모델을 오토 모델에 등록
model_path = 'rnn-config'
AutoConfig.register(model_path, RnnConfig)
AutoModelForTimeSeriesRegression.register(RnnConfig, RnnForTimeSeriesRegression)
'''
